"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var css_1 = __importDefault(require("css"));
var DEFAULT_MATCH_FILTER = { media: undefined, target: undefined };
var kebabCase = function (str) {
    return str
        .replace(/([a-z])([A-Z])/g, '$1-$2')
        .replace(/\s+/g, '-')
        .toLowerCase();
};
var removeSpaces = function (str) { return str && str.replace(/\s/g, ''); };
var mapProperties = function (properties) {
    return Object.keys(properties).map(function (property) { return kebabCase(property) + ":" + properties[property]; });
};
var getMountedProperties = function () {
    return Array.from(document.styleSheets)
        .map(function (sheet) {
        // @ts-ignore
        return sheet.cssRules.map(function (rule) { return rule.style.cssText; });
    })
        .join(' ');
};
var onlyRules = function (rules) { var _a; return (_a = rules) === null || _a === void 0 ? void 0 : _a.filter(function (r) { return r.type === 'rule'; }); };
var findMediaRules = function (allRules, media) {
    if (allRules === void 0) { allRules = []; }
    for (var _i = 0, allRules_1 = allRules; _i < allRules_1.length; _i++) {
        var rule = allRules_1[_i];
        if (!rule)
            return;
        if ('media' in rule) {
            if (removeSpaces(rule.media) === removeSpaces(media) && 'rules' in rule)
                return rule.rules;
            if ('rules' in rule)
                return findMediaRules(rule.rules, media);
        }
    }
    return;
};
var getRules = function (ast, filter, className) {
    var _a;
    var media = filter.media, target = filter.target;
    // rules are present directly inside ast.stylesheet.rules
    // but if a media query is present it is nested inside ast.stylesheet.media.rules
    // this inner function returns the relevant rules
    var getAllRules = function () {
        var _a, _b;
        if (media) {
            return onlyRules(findMediaRules((_a = ast.stylesheet) === null || _a === void 0 ? void 0 : _a.rules, media));
        }
        return (_b = ast.stylesheet) === null || _b === void 0 ? void 0 : _b.rules.filter(function (r) { return (r.type = 'rule'); }); // omit media objects
    };
    var allRules = getAllRules();
    var klass = target ? "." + className + target : "." + className;
    return (_a = allRules) === null || _a === void 0 ? void 0 : _a.filter(function (r) {
        var _a;
        if ('selectors' in r) {
            return (_a = r.selectors) === null || _a === void 0 ? void 0 : _a.find(function (s) { return removeSpaces(s) === removeSpaces(klass); });
        }
        return;
    });
};
var findStylesInRules = function (styles, rules) {
    var found = [];
    if (!rules)
        return found;
    styles.forEach(function (s) {
        var _a;
        (_a = rules) === null || _a === void 0 ? void 0 : _a.forEach(function (r) {
            var _a;
            if ('declarations' in r) {
                (_a = r.declarations) === null || _a === void 0 ? void 0 : _a.forEach(function (d) {
                    if ('property' in d) {
                        if (s === d.property + ":" + d.value)
                            found.push(s);
                    }
                });
            }
        });
    });
    return found;
};
function toHaveCompiledCss(element) {
    var _a;
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    var property = args[0], value = args[1], _b = args[2], matchFilter = _b === void 0 ? DEFAULT_MATCH_FILTER : _b;
    var properties = typeof property === 'string' ? (_a = {}, _a[property] = value, _a) : property;
    var inlineStyleTag = element.parentElement && element.parentElement.querySelector('style');
    var styleElements = inlineStyleTag != null ? [inlineStyleTag] : Array.from(document.head.querySelectorAll('style'));
    if (!styleElements) {
        return {
            pass: false,
            message: function () { return 'pairing style element was not found'; },
        };
    }
    var stylesToFind = mapProperties(properties);
    var foundStyles = [];
    var classNames = element.className.split(' ');
    var _loop_1 = function (styleElement) {
        var css = styleElement.textContent || '';
        // This is a hack to get ahold of the styles.
        // Unfortunately JSDOM doesn't handle css variables properly
        // See: https://github.com/jsdom/jsdom/issues/1895
        // @ts-ignore
        var styles = element[Object.keys(element)[0]].memoizedProps.style;
        if (styles && Object.keys(styles).length > 0) {
            Object.entries(styles).forEach(function (_a) {
                var key = _a[0], value = _a[1];
                // Replace all instances of var with the value.
                // We split and join to replace all instances without needing to jump into a dynamic regex.
                css = css.split("var(" + key + ")").join(value);
            });
        }
        var ast = css_1.default.parse(css);
        classNames.forEach(function (c) {
            var rules = getRules(ast, matchFilter, c);
            foundStyles.push.apply(foundStyles, findStylesInRules(stylesToFind, rules));
        });
    };
    for (var _c = 0, styleElements_1 = styleElements; _c < styleElements_1.length; _c++) {
        var styleElement = styleElements_1[_c];
        _loop_1(styleElement);
    }
    var notFoundStyles = stylesToFind.filter(function (style) { return !foundStyles.includes(style); });
    var foundFormatted = stylesToFind.join(', ');
    var notFoundFormatted = notFoundStyles.join(', ');
    if (foundStyles.length > 0 && notFoundStyles.length === 0) {
        return {
            pass: true,
            message: !this.isNot
                ? function () { return ''; }
                : function () {
                    return "Found \"" + foundFormatted + "\" on <" + element.nodeName.toLowerCase() + " css={`" + getMountedProperties() + "`}> element.";
                },
        };
    }
    return {
        pass: false,
        message: function () {
            return "Could not find \"" + notFoundFormatted + "\" on <" + element.nodeName.toLowerCase() + " css={`" + getMountedProperties() + "`}> element.";
        },
    };
}
exports.toHaveCompiledCss = toHaveCompiledCss;
//# sourceMappingURL=matchers.js.map